Subject: bug#50946: insert-file-contents can corrupt buffers. [Was: bug#50946: Emacs-28: Inadequate coding in hack-elisp-shorthands]

From: João Távora <[email-masked]>
Date: Sun, 03 Oct 2021 18:05:33 +0100

Eli Zaretskii <[email-masked]> writes:

>> No, fboundp also works, probably.  Do you prefer that to a hook?
>> A hook is, in theory, more powerful.
> Well, using a hook in our own sources is IMO ... "icky".
> So yes, I prefer the fboundp test.  And then shorthands.el will deal

Hmm, I think we do use them a lot.  Or at least foo-function variables
for the same purpose.

Icky or not, the fboundp one isn't working, the hook one I gave you
earlier does.

With the fboundp, I get some recursive load error (not the first time).
I didn't investigate, maybe you can tell what's going on?  I am missing
sometehing obvious?

After my sig is the patch, followed by the error (search for "make
error".

João

The patch:

diff --git a/lisp/loadup.el b/lisp/loadup.el
index 3fb6b81328..3a55d2c805 100644
--- a/lisp/loadup.el
+++ b/lisp/loadup.el
@@ -355,7 +355,6 @@
 (load "paren")
 
 (load "shorthands")
-(setq load-source-file-function #'load-with-shorthands-and-code-conversion)
 
 (load "emacs-lisp/eldoc")
 (load "cus-start") ;Late to reduce customize-rogue (needs loaddefs.el anyway)
diff --git a/lisp/shorthands.el b/lisp/shorthands.el
index c31ef3d216..adbbf3713b 100644
--- a/lisp/shorthands.el
+++ b/lisp/shorthands.el
@@ -25,40 +25,8 @@
 ;; Basic helpers for loading files with Shorthands.
 
 ;;; Code:
-(require 'files)
 (eval-when-compile (require 'cl-lib))
 
-(defun hack-read-symbol-shorthands (fullname)
-  "Return value of `read-symbol-shorthands' file-local variable in FULLNAME.
-FULLNAME is the absolute file name of an Elisp .el file which
-potentially specifies a file-local value for
-`read-symbol-shorthands'.  The Elisp code in FULLNAME isn't read
-or evaluated in any way, except for extraction of the
-buffer-local value of `read-symbol-shorthands'."
-  (let* ((size (nth 7 (file-attributes fullname)))
-         (from (max 0 (- size 3000)))
-         (to size))
-    (with-temp-buffer
-      (while (and (< (buffer-size) 3000) (>= from 0))
-        (insert-file-contents fullname nil from to)
-        (setq to from
-              from (cond
-                    ((= from 0) -1)
-                    (t (max 0 (- from 100))))))
-      ;; FIXME: relies on the `hack-local-variables--find-variables'
-      ;; detail of files.el.  That function should be exported,
-      ;; possibly be refactored into two parts, since we're only
-      ;; interested in basic "Local Variables" parsing.
-      (alist-get 'read-symbol-shorthands (hack-local-variables--find-variables)))))
-
-(defun load-with-shorthands-and-code-conversion (fullname file noerror nomessage)
-  "Like `load-with-code-conversion', but also consider Elisp shorthands.
-This function uses shorthands defined in the file FULLNAME's local
-value of `read-symbol-shorthands', when it processes that file's Elisp code."
-  (let ((read-symbol-shorthands (hack-read-symbol-shorthands fullname)))
-    (load-with-code-conversion fullname file noerror nomessage)))
-
-
 ;; FIXME: move this all to progmodes/elisp-mode.el?  OTOH it'd make
 ;; more sense there, OTOH all the elisp font-lock stuff is actually in
 ;; lisp/emacs-lisp/lisp-mode.el, which isn't right either.  So

The make error I get:

make -C lib all
make[1]: Entering directory '/home/capitaomorte/Source/Emacs/emacs/lib'
make[1]: Nothing to be done for 'all'.
make[1]: Leaving directory '/home/capitaomorte/Source/Emacs/emacs/lib'
make -C lib-src all
make[1]: Entering directory '/home/capitaomorte/Source/Emacs/emacs/lib-src'
make[1]: Nothing to be done for 'all'.
make[1]: Leaving directory '/home/capitaomorte/Source/Emacs/emacs/lib-src'
make -C src VCSWITNESS='$(srcdir)/../.git/logs/HEAD' BIN_DESTDIR=''/usr/local/bin/'' \
	 ELN_DESTDIR='/usr/local/lib/emacs/28.0.60/' all
make[1]: Entering directory '/home/capitaomorte/Source/Emacs/emacs/src'
make -C ../admin/charsets all
make[2]: Entering directory '/home/capitaomorte/Source/Emacs/emacs/admin/charsets'
make[2]: Nothing to be done for 'all'.
make[2]: Leaving directory '/home/capitaomorte/Source/Emacs/emacs/admin/charsets'
make -C ../admin/unidata charscript.el
make[2]: Entering directory '/home/capitaomorte/Source/Emacs/emacs/admin/unidata'
make[2]: Nothing to be done for 'charscript.el'.
make[2]: Leaving directory '/home/capitaomorte/Source/Emacs/emacs/admin/unidata'
make -C ../admin/unidata emoji-zwj.el
make[2]: Entering directory '/home/capitaomorte/Source/Emacs/emacs/admin/unidata'
make[2]: Nothing to be done for 'emoji-zwj.el'.
make[2]: Leaving directory '/home/capitaomorte/Source/Emacs/emacs/admin/unidata'
make -C ../lisp autoloads EMACS="../src/bootstrap-emacs"
make[2]: Entering directory '/home/capitaomorte/Source/Emacs/emacs/lisp'
make -C ../leim all EMACS="../src/bootstrap-emacs"
make[3]: Entering directory '/home/capitaomorte/Source/Emacs/emacs/leim'
make[3]: Nothing to be done for 'all'.
make[3]: Leaving directory '/home/capitaomorte/Source/Emacs/emacs/leim'
make -C ../admin/grammars all EMACS="../../src/bootstrap-emacs"
make[3]: Entering directory '/home/capitaomorte/Source/Emacs/emacs/admin/grammars'
make[3]: Nothing to be done for 'all'.
make[3]: Leaving directory '/home/capitaomorte/Source/Emacs/emacs/admin/grammars'
  GEN      loaddefs.el
  SCRAPE   . ./calc ./calendar ./cedet ./cedet/ede ./cedet/semantic ...
  SCRAPE   ./cedet/semantic/analyze ./cedet/semantic/bovine ...
  SCRAPE   ./cedet/semantic/decorate ./cedet/semantic/symref ...
  SCRAPE   ./cedet/semantic/wisent ./cedet/srecode ./emacs-lisp ./emulation ...
  SCRAPE   ./erc ./eshell ./gnus ./image ./international ./language ./leim ...
  SCRAPE   ./leim/ja-dic ./leim/quail ./mail ./mh-e ./net ./nxml ./org ...
  SCRAPE   ./play ./progmodes ./textmodes ./url ./vc
  INFO     Scraping files for loaddefs.el... 
  INFO     Scraping files for loaddefs.el...done
make[2]: Leaving directory '/home/capitaomorte/Source/Emacs/emacs/lisp'
make -C ../admin/unidata all EMACS="../../src/bootstrap-emacs"
make[2]: Entering directory '/home/capitaomorte/Source/Emacs/emacs/admin/unidata'
make[2]: Nothing to be done for 'all'.
make[2]: Leaving directory '/home/capitaomorte/Source/Emacs/emacs/admin/unidata'
make -C ../admin/charsets cp51932.el
make[2]: Entering directory '/home/capitaomorte/Source/Emacs/emacs/admin/charsets'
make[2]: Nothing to be done for 'cp51932.el'.
make[2]: Leaving directory '/home/capitaomorte/Source/Emacs/emacs/admin/charsets'
make -C ../admin/charsets eucjp-ms.el
make[2]: Entering directory '/home/capitaomorte/Source/Emacs/emacs/admin/charsets'
make[2]: Nothing to be done for 'eucjp-ms.el'.
make[2]: Leaving directory '/home/capitaomorte/Source/Emacs/emacs/admin/charsets'
LC_ALL=C ./temacs -batch  -l loadup --temacs=pdump \
	--bin-dest /usr/local/bin/ --eln-dest /usr/local/lib/emacs/28.0.60/
Loading loadup.el (source)...
Dump mode: pdump
Using load-path (/home/capitaomorte/Source/Emacs/emacs/lisp)
Loading emacs-lisp/byte-run...
Loading emacs-lisp/backquote...
Loading subr...
Loading version...
Loading widget...
Loading custom...
Loading emacs-lisp/map-ynp...
Loading international/mule...
Loading international/mule-conf...
Loading env...
Loading format...
Loading bindings...
Loading window...
Loading files...
Loading emacs-lisp/macroexp...
Loading cus-face...
Loading faces...
Loading loaddefs.el (source)...
Loading button...
Loading emacs-lisp/nadvice...
Loading emacs-lisp/cl-preloaded...
Loading obarray...
Loading abbrev...
Loading simple...
Loading help...
Loading jka-cmpr-hook...
Loading epa-hook...
Loading international/mule-cmds...
Loading case-table...
Loading international/charprop.el (source)...
Loading international/characters...
Recursive load: "/home/capitaomorte/Source/Emacs/emacs/lisp/international/uni-special-lowercase.el", "/home/capitaomorte/Source/Emacs/emacs/lisp/international/uni-special-lowercase.el", "/home/capitaomorte/Source/Emacs/emacs/lisp/international/uni-special-lowercase.el", "/home/capitaomorte/Source/Emacs/emacs/lisp/international/uni-special-lowercase.el", "/home/capitaomorte/Source/Emacs/emacs/lisp/international/uni-special-lowercase.el", "/home/capitaomorte/Source/Emacs/emacs/lisp/international/uni-bidi.el", "/home/capitaomorte/Source/Emacs/emacs/lisp/international/characters.elc", "/home/capitaomorte/Source/Emacs/emacs/lisp/loadup.el"
make[1]: *** [Makefile:587: emacs.pdmp] Error 255
make[1]: Leaving directory '/home/capitaomorte/Source/Emacs/emacs/src'
make: *** [Makefile:449: src] Error 2
diff --git a/lisp/international/mule.el b/lisp/international/mule.el
index 2a855b5673..11d344433f 100644
--- a/lisp/international/mule.el
+++ b/lisp/international/mule.el
@@ -319,7 +319,8 @@ load-with-code-conversion
 	  (let ((load-true-file-name fullname)
                 (load-file-name fullname)
                 (set-auto-coding-for-load t)
-		(inhibit-file-name-operation nil))
+		(inhibit-file-name-operation nil)
+                (shorthands))
 	    (with-current-buffer buffer
               ;; So that we don't get completely screwed if the
               ;; file is encoded in some complicated character set,
@@ -328,6 +329,10 @@ load-with-code-conversion
 	      ;; Don't let deactivate-mark remain set.
 	      (let (deactivate-mark)
 		(insert-file-contents fullname))
+              (setq shorthands
+                    (and (fboundp 'hack-local-variables--find-variables)
+                         (alist-get 'read-symbol-shorthands
+                                    (hack-local-variables--find-variables))))
 	      ;; If the loaded file was inserted with no-conversion or
 	      ;; raw-text coding system, make the buffer unibyte.
 	      ;; Otherwise, eval-buffer might try to interpret random
@@ -338,11 +343,13 @@ load-with-code-conversion
 		  (set-buffer-multibyte nil))
 	      ;; Make `kill-buffer' quiet.
 	      (set-buffer-modified-p nil))
-	    ;; Have the original buffer current while we eval.
-	    (eval-buffer buffer nil
-			 ;; This is compatible with what `load' does.
-                         (if dump-mode file fullname)
-			 nil t))
+	    ;; Have the original buffer current while we eval,
+            ;; but consider shorthands of the eval'ed one.
+	    (let ((read-symbol-shorthands shorthands))
+              (eval-buffer buffer nil
+			   ;; This is compatible with what `load' does.
+                           (if dump-mode file fullname)
+			   nil t)))
 	(let (kill-buffer-hook kill-buffer-query-functions)
 	  (kill-buffer buffer)))
       (do-after-load-evaluation fullname)